#include <iostream>
#include <boost/format.hpp>
#include <boost/program_options.hpp>
#include "store.hpp"
#include "extract.hpp"
#include "custom_resolver.hpp"
using namespace std;
using namespace boost;
namespace po = boost::program_options;

int main(int argc, char* argv[]) {
    TagMachine::WavPackFileTypeResolver::install();
    
    po::options_description op_general("Options");
    op_general.add_options()
        ("help,h", "print this help.")

        ("store,s",
         po::value<string>(),
         "read a tag file and store it into a music file.")

        ("extract,e",
         po::value<string>(),
         "extract tags from a music file and write them to a tag file.")

        ("format,f",
         po::value<string>()->default_value("vorbistag"),
         "format of the tag file: only `vorbistag' is currently supported.")
        ;

    po::options_description op_hidden("Hidden");
    op_hidden.add_options()
        ("music-file", po::value<string>())
        ;

    po::positional_options_description op_positional;
    op_positional.add("music-file", 1);

    po::options_description all;
    all.add(op_general);
    all.add(op_hidden);

    po::options_description help;
    help.add(op_general);

    po::variables_map vm;
    try {
        po::store(
            po::command_line_parser(argc, argv).options(all).positional(op_positional).run(),
            vm);
        po::notify(vm);

        if (vm.count("help")) {
            cerr << format("Usage: %s MUSIC-FILE --store=TAG-FILE [options]") % argv[0] << endl;
            cerr << format("       %s MUSIC-FILE --extract=TAG-FILE [options]") % argv[0] << endl;
            cerr << help << endl;
            return 1;
        }

        if (vm.count("music-file") == 1) {
            if (vm.count("store") == 1 && vm.count("extract") == 0) {
                TagMachine::store_tag(
                    vm["music-file"].as<string>(),
                    vm["store"     ].as<string>(),
                    vm["format"    ].as<string>());
                return 0;
            }
            else if (vm.count("store") == 0 && vm.count("extract") == 1) {
                TagMachine::extract_tag(
                    vm["music-file"].as<string>(),
                    vm["extract"   ].as<string>(),
                    vm["format"    ].as<string>());
                return 0;
            }
            else if (vm.count("store") == 0 && vm.count("extract") == 0) {
                TagMachine::extract_tag(
                    vm["music-file"].as<string>(),
                    "-",
                    vm["format"    ].as<string>());
                return 0;
            }
        }

        throw (format("parameter error. Try `%s --help'") % argv[0]).str();
    }
    catch (const std::exception& e) {
        cerr << format("%s: %s") % argv[0] % e.what() << endl;
        return 1;
    }
    catch (const std::string& e) {
        cerr << format("%s: %s") % argv[0] % e << endl;
        return 1;
    }

    return 0;
}
